package com.zskk.controller;

import java.security.AlgorithmParameters;
import java.security.Key;
import java.util.Date;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import com.jfinal.aop.Duang;
import com.jfinal.kit.Base64Kit;
import com.jfinal.kit.Kv;
import com.jfinal.kit.StrKit;
import com.jfinal.weixin.sdk.api.ApiConfigKit;
import com.jfinal.weixin.sdk.api.ApiResult;
import com.jfinal.weixin.sdk.cache.IAccessTokenCache;
import com.jfinal.weixin.sdk.encrypt.WxaBizDataCrypt;
import com.jfinal.weixin.sdk.utils.Charsets;
import com.jfinal.wxaapp.api.WxaUserApi;
import com.jfinal.wxaapp.jfinal.WxaController;
import com.zskk.controller.bean.LoginBean;
import com.zskk.controller.bean.ResultBean;
import com.zskk.model.Doctors;
import com.zskk.model.DoctorsWechat;
import com.zskk.util.ErrorConstant;

/**
 * 微信小程序用户api接口
 * @author L.cm
 */
public class WxaUserApiController extends WxaController {
	/**
	 * 登陆接口
	 */
	public void login() {
		String jsCode = getPara("code");
		if (StrKit.isBlank(jsCode)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "code is blank");
			renderJson(ErrorConstant.ERROR_WXA_CODE_BLANK);
			return;
		}
		// 获取SessionKey
		WxaUserApi wxaUserApi = Duang.duang(WxaUserApi.class);
		ApiResult apiResult = wxaUserApi.getSessionKey(jsCode);
		// 返回{"session_key":"nzoqhc3OnwHzeTxJs+inbQ==","expires_in":2592000,"openid":"oVBkZ0aYgDMDIywRdgPW8-joxXc4"}
		if (!apiResult.isSucceed()) {
//			JSONObject object = new JSONObject();
//			object.put("sessionId", sessionId);
			ResultBean resultBean = new ResultBean(apiResult.getInt("errcode"),apiResult.getStr("errmsg"));
			renderJson(resultBean);
			return;
		}
		// 利用 appId 与 accessToken 建立关联，支持多账户
		IAccessTokenCache accessTokenCache = ApiConfigKit.getAccessTokenCache();
		String sessionId = StrKit.getRandomUUID();
		DoctorsWechat doctorsWechatfind = DoctorsWechat.dao.findFirst("select * from doctors_wechat where wxa_openid=?", apiResult.getStr("openid"));
		if (doctorsWechatfind == null) {
			DoctorsWechat doctorsWechat = new DoctorsWechat();
			doctorsWechat.setWxaOpenid(apiResult.getStr("openid"));
			if (apiResult.getStr("unionid") != null) {
				doctorsWechat.setUnionid(apiResult.getStr("unionid"));
			}
			doctorsWechat.setCreateAt(new Date());
			doctorsWechat.setUpdateAt(new Date());
			doctorsWechat.save();
		}else {
			doctorsWechatfind.setUpdateAt(new Date());
			doctorsWechatfind.update();
		}
		Doctors doctors = Doctors.dao.findById(doctorsWechatfind.getDoctorId());
		accessTokenCache.set("wxa:session:" + sessionId, apiResult.getJson());
		LoginBean loginBean = new LoginBean();
		loginBean.setSessionId(sessionId);
		loginBean.setDoctorBean(doctors);
		renderJson(new ResultBean(loginBean));
//		renderJson("sessionId", sessionId);
	}
	
	public void loginBySession() {
		IAccessTokenCache accessTokenCache = ApiConfigKit.getAccessTokenCache();
//		String sessionId = getHeader("wxa-sessionid");
		String sessionId = getPara("wxa-sessionid");
		if (StrKit.isBlank(sessionId)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "wxa_session Header is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK);
			return;
		}
		String sessionJson = accessTokenCache.get("wxa:session:" + sessionId);
		if (StrKit.isBlank(sessionJson)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "wxa_session sessionJson is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK);
			return;
		}
		ApiResult sessionResult = ApiResult.create(sessionJson);
		// 获取sessionKey
		String sessionKey = sessionResult.get("session_key");
		if (StrKit.isBlank(sessionKey)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "sessionKey is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK);
			return;
		}
		DoctorsWechat doctorsWechat = DoctorsWechat.dao.findFirst("select * from doctors_wechat where wxa_openid=?", sessionResult.get("openid"));
		Doctors doctors = Doctors.dao.findById(doctorsWechat.getDoctorId());
		LoginBean loginBean = new LoginBean();
		loginBean.setSessionId(sessionId);
		loginBean.setDoctorBean(doctors);
		renderJson(new ResultBean(loginBean));
	}
	
	/**
	 * 服务端解密用户信息接口
	 * 获取unionId
	 */
	public void info() {
		String signature = getPara("signature");
		String rawData = getPara("rawData");
		
		String encryptedData = getPara("encryptedData");
		String iv = getPara("iv");
		
		// 参数空校验 不做演示
		// 利用 appId 与 accessToken 建立关联，支持多账户
		IAccessTokenCache accessTokenCache = ApiConfigKit.getAccessTokenCache();
		String sessionId = getHeader("wxa-sessionid");
		if (StrKit.isBlank(sessionId)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "wxa_session Header is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK);
			return;
		}
		String sessionJson = accessTokenCache.get("wxa:session:" + sessionId);
		if (StrKit.isBlank(sessionJson)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "wxa_session sessionJson is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK);
			return;
		}
		ApiResult sessionResult = ApiResult.create(sessionJson);
		// 获取sessionKey
		String sessionKey = sessionResult.get("session_key");
		if (StrKit.isBlank(sessionKey)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "sessionKey is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK);
			return;
		}
		// 用户信息校验
		WxaUserApi wxaUserApi = Duang.duang(WxaUserApi.class);
		boolean check = wxaUserApi.checkUserInfo(sessionKey, rawData, signature);
		if (!check) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "UserInfo check fail");
			renderJson(ErrorConstant.ERROR_USERINFO_CHECK_FAIL);
			return;
		}
		// 服务端解密用户信息
		ApiResult apiResult = wxaUserApi.getUserInfo(sessionKey, encryptedData, iv);
		if (!apiResult.isSucceed()) {
			ResultBean resultBean = new ResultBean(apiResult.getInt("errcode"),apiResult.getStr("errmsg"));

			renderJson(resultBean);
			return;
		}
		// 如果开发者拥有多个移动应用、网站应用、和公众帐号（包括小程序），可通过unionid来区分用户的唯一性
		// 同一用户，对同一个微信开放平台下的不同应用，unionid是相同的。
//		String unionId = apiResult.get("unionId");
		String phone = apiResult.getStr("purePhoneNumber");
		if (StrKit.isBlank(phone)) {
			renderJson(ErrorConstant.ERROR_PHONE_BLANK);
			return;
		}
		Doctors doctors = Doctors.dao.findFirst("select * from doctors where phone=?", phone);
		if (doctors == null) {
			renderJson(ErrorConstant.ERROR_DOCTOR_NOT_FIND);
			return;
		}
		DoctorsWechat doctorsWechat = DoctorsWechat.dao.findFirst("select * from doctors_wechat where wxa_openid=?", sessionResult.get("openid"));
		doctorsWechat.setDoctorId(doctors.getId());
		doctorsWechat.update();
		LoginBean loginBean = new LoginBean();
		loginBean.setSessionId(sessionId);
		loginBean.setDoctorBean(doctors);
		renderJson(new ResultBean(loginBean));
	}

	/**
	 * 服务端解密用户信息接口
	 * 获取unionId
	 */
	public void phoneLogin() {
		
		String encryptedData = getPara("encryptedData");
		String iv = getPara("iv");
		String sessionId = getPara("sessionid");
		// 参数空校验 不做演示
		if (StrKit.isBlank(encryptedData) || StrKit.isBlank(iv)) {
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK);
			return;
		}
		// 利用 appId 与 accessToken 建立关联，支持多账户
		IAccessTokenCache accessTokenCache = ApiConfigKit.getAccessTokenCache();
		if (StrKit.isBlank(sessionId)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "wxa_session Header is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK);
			return;
		}
		String sessionJson = accessTokenCache.get("wxa:session:" + sessionId);
		if (StrKit.isBlank(sessionJson)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "wxa_session sessionJson is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK2);
			return;
		}

		ApiResult sessionResult = ApiResult.create(sessionJson);
		// 获取sessionKey
		String sessionKey = sessionResult.get("session_key");
		if (StrKit.isBlank(sessionKey)) {
			Kv data = Kv.by("errcode", 500)
					.set("errmsg", "sessionKey is blank");
			renderJson(ErrorConstant.ERROR_WXA_SESSION_BLANK3);
			return;
		}

//		try {	
		
		// 用户信息校验
		WxaUserApi wxaUserApi = Duang.duang(WxaUserApi.class);
		// 服务端解密用户信息
//		renderText(sessionKey+"    "+encryptedData + "     "+ iv);

	
//			WxaBizDataCrypt dataCrypt = new WxaBizDataCrypt(sessionKey);
			byte[] bizData = Base64Kit.decode(encryptedData);
	        byte[] keyByte = Base64Kit.decode(sessionKey);
	        byte[] ivByte  = Base64Kit.decode(iv);
	        try {
	            Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
	            Key sKeySpec = new SecretKeySpec(keyByte, "AES");
//	            // 初始化
	            AlgorithmParameters params = AlgorithmParameters.getInstance("AES");
	            params.init(new IvParameterSpec(ivByte));
	            cipher.init(Cipher.DECRYPT_MODE, sKeySpec, params);
	            byte[] original = cipher.doFinal(bizData);
	            // 去除补位字符
	        } catch (Exception e) {
				renderText("aes");

	        }
			renderText("aa");

	          
//	        			ApiResult apiResult = wxaUserApi.getUserInfo(sessionKey, encryptedData, iv);
//			renderText(json);

		
//		if (!apiResult.isSucceed()) {
//			ResultBean resultBean = new ResultBean(apiResult.getInt("errcode"),apiResult.getStr("errmsg"));
//			renderJson(resultBean);
//			return;
//		}
		// 如果开发者拥有多个移动应用、网站应用、和公众帐号（包括小程序），可通过unionid来区分用户的唯一性
		// 同一用户，对同一个微信开放平台下的不同应用，unionid是相同的。
		
//			String phone = apiResult.getStr("purePhoneNumber");
//			if (StrKit.isBlank(phone)) {
//				renderJson(ErrorConstant.ERROR_PHONE_BLANK);
//				return;
//			}
//			renderNull();

//			Doctors doctors = Doctors.dao.findFirst("select * from doctors where phone=?", phone);
//			if (doctors == null) {
//				renderJson(ErrorConstant.ERROR_DOCTOR_NOT_FIND);
//				return;
//			}
//			DoctorsWechat doctorsWechat = DoctorsWechat.dao.findFirst("select * from doctors_wechat where wxa_openid=?", sessionResult.get("openid"));
//			doctorsWechat.setDoctorId(doctors.getId());
//			doctorsWechat.update();
//			LoginBean loginBean = new LoginBean();
//			loginBean.setSessionId(sessionId);
//			loginBean.setDoctorBean(doctors);
//			renderJson(new ResultBean(loginBean));
//		} catch (Exception e) {
//			// TODO: handle exception
//			renderText(e.toString());
//		}
		
	}
}
